@@ -41,18 +41,26 @@ impl std::fmt::Display for DependencyCategory {
 }
 
 #[derive(Debug)]
+/// Error types for build system operations.
+///
+/// These represent different kinds of errors that can occur when working with build systems.
 pub enum Error {
     /// The build system could not be detected.
     NoBuildSystemDetected,
 
+    /// Error occurred while installing dependencies.
     DependencyInstallError(InstallerError),
 
+    /// Error detected and analyzed from build output.
     Error(crate::analyze::AnalyzedError),
 
+    /// Error from an IO operation.
     IoError(std::io::Error),
 
+    /// The requested operation is not implemented by this build system.
     Unimplemented,
 
+    /// Generic error with a message.
     Other(String),
 }
 
@@ -174,13 +182,21 @@ impl std::fmt::Display for Error {
 impl std::error::Error for Error {}
 
 #[derive(Debug, Clone)]
+/// Target configuration for installation.
+///
+/// Defines where and how packages should be installed.
 pub struct InstallTarget {
+    /// The scope of installation (e.g., global or user).
     pub scope: InstallationScope,
 
+    /// Optional installation prefix directory.
     pub prefix: Option<PathBuf>,
 }
 
 impl DependencyCategory {
+    /// Get all standard dependency categories.
+    ///
+    /// Returns an array containing all standard dependency categories.
     pub fn all() -> [DependencyCategory; 5] {
         [
             DependencyCategory::Universal,
@@ -193,10 +209,17 @@ impl DependencyCategory {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
+/// Standard build system actions.
+///
+/// These represent the common actions that can be performed by build systems.
 pub enum Action {
+    /// Clean the build environment.
     Clean,
+    /// Build the project.
     Build,
+    /// Run the project's tests.
     Test,
+    /// Install the project.
     Install,
 }
 
@@ -220,6 +243,17 @@ pub trait BuildSystem: std::fmt::Debug {
     /// The name of the buildsystem.
     fn name(&self) -> &str;
 
+    /// Create a distribution package for the project.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `installer` - Installer to use for installing dependencies
+    /// * `target_directory` - Directory where the distribution package should be created
+    /// * `quiet` - Whether to suppress output
+    ///
+    /// # Returns
+    /// * The filename of the created distribution package on success
+    /// * Error if the distribution creation fails
     fn dist(
         &self,
         session: &dyn Session,
@@ -228,6 +262,18 @@ pub trait BuildSystem: std::fmt::Debug {
         quiet: bool,
     ) -> Result<std::ffi::OsString, Error>;
 
+    /// Install the dependencies declared by the build system.
+    ///
+    /// # Arguments
+    /// * `categories` - The categories of dependencies to install
+    /// * `scopes` - The scopes in which to install the dependencies
+    /// * `session` - The session to run commands in
+    /// * `installer` - Installer to use for installing dependencies
+    /// * `fixers` - Optional list of fixers to use if getting dependency information fails
+    ///
+    /// # Returns
+    /// * `Ok(())` if the dependencies were installed successfully
+    /// * Error if installation fails
     fn install_declared_dependencies(
         &self,
         categories: &[DependencyCategory],
@@ -237,31 +283,60 @@ pub trait BuildSystem: std::fmt::Debug {
         fixers: Option<&[&dyn crate::fix_build::BuildFixer<InstallerError>]>,
     ) -> Result<(), Error> {
         let declared_deps = self.get_declared_dependencies(session, fixers)?;
-        let relevant = declared_deps
+        let relevant: Vec<_> = declared_deps
             .into_iter()
             .filter(|(c, _d)| categories.contains(c))
             .map(|(_, d)| d)
-            .collect::<Vec<_>>();
+            .collect();
         log::debug!("Declared dependencies: {:?}", relevant);
-        install_missing_deps(
-            session,
-            installer,
-            scopes,
-            relevant
-                .iter()
-                .map(|d| d.as_ref())
-                .collect::<Vec<_>>()
-                .as_slice(),
-        )?;
+        let dep_refs: Vec<&dyn Dependency> = relevant.iter().map(|d| d.as_ref()).collect();
+        install_missing_deps(session, installer, scopes, &dep_refs)?;
         Ok(())
     }
 
+    /// Run tests for the project.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `installer` - Installer to use for installing dependencies
+    ///
+    /// # Returns
+    /// * `Ok(())` if the tests pass
+    /// * Error if the tests fail
     fn test(&self, session: &dyn Session, installer: &dyn Installer) -> Result<(), Error>;
 
+    /// Build the project.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `installer` - Installer to use for installing dependencies
+    ///
+    /// # Returns
+    /// * `Ok(())` if the build succeeds
+    /// * Error if the build fails
     fn build(&self, session: &dyn Session, installer: &dyn Installer) -> Result<(), Error>;
 
+    /// Clean the project's build artifacts.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `installer` - Installer to use for installing dependencies
+    ///
+    /// # Returns
+    /// * `Ok(())` if the clean succeeds
+    /// * Error if the clean fails
     fn clean(&self, session: &dyn Session, installer: &dyn Installer) -> Result<(), Error>;
 
+    /// Install the project.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `installer` - Installer to use for installing dependencies
+    /// * `install_target` - Target configuration for the installation
+    ///
+    /// # Returns
+    /// * `Ok(())` if the installation succeeds
+    /// * Error if the installation fails
     fn install(
         &self,
         session: &dyn Session,
@@ -269,34 +344,172 @@ pub trait BuildSystem: std::fmt::Debug {
         install_target: &InstallTarget,
     ) -> Result<(), Error>;
 
+    /// Get the dependencies declared by the build system.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `fixers` - Optional list of fixers to use if getting dependency information fails
+    ///
+    /// # Returns
+    /// * List of dependencies with their categories
+    /// * Error if getting dependency information fails
     fn get_declared_dependencies(
         &self,
         session: &dyn Session,
         fixers: Option<&[&dyn crate::fix_build::BuildFixer<InstallerError>]>,
     ) -> Result<Vec<(DependencyCategory, Box<dyn Dependency>)>, Error>;
 
+    /// Get the outputs declared by the build system.
+    ///
+    /// # Arguments
+    /// * `session` - The session to run commands in
+    /// * `fixers` - Optional list of fixers to use if getting output information fails
+    ///
+    /// # Returns
+    /// * List of declared outputs
+    /// * Error if getting output information fails
     fn get_declared_outputs(
         &self,
         session: &dyn Session,
         fixers: Option<&[&dyn crate::fix_build::BuildFixer<InstallerError>]>,
     ) -> Result<Vec<Box<dyn Output>>, Error>;
 
+    /// Convert this build system to Any for dynamic casting.
+    ///
+    /// This method allows for conversion of the build system to concrete types at runtime.
+    ///
+    /// # Returns
+    /// A reference to this build system as Any
     fn as_any(&self) -> &dyn std::any::Any;
 }
 
+/// A single buildsystem registry entry.
+struct BuildSystemEntry {
+    /// The name of the buildsystem
+    name: &'static str,
+    /// Function to probe for this buildsystem
+    probe: fn(&Path) -> Option<Box<dyn BuildSystem>>,
+}
+
+/// Registry of all supported buildsystems in detection order.
+const BUILDSYSTEM_REGISTRY: &[BuildSystemEntry] = &[
+    BuildSystemEntry {
+        name: "pear",
+        probe: Pear::probe,
+    },
+    BuildSystemEntry {
+        name: "setup.py",
+        probe: crate::buildsystems::python::SetupPy::probe,
+    },
+    BuildSystemEntry {
+        name: "node",
+        probe: crate::buildsystems::node::Node::probe,
+    },
+    BuildSystemEntry {
+        name: "waf",
+        probe: crate::buildsystems::waf::Waf::probe,
+    },
+    BuildSystemEntry {
+        name: "gem",
+        probe: crate::buildsystems::ruby::Gem::probe,
+    },
+    BuildSystemEntry {
+        name: "meson",
+        probe: crate::buildsystems::meson::Meson::probe,
+    },
+    BuildSystemEntry {
+        name: "cargo",
+        probe: crate::buildsystems::rust::Cargo::probe,
+    },
+    BuildSystemEntry {
+        name: "cabal",
+        probe: crate::buildsystems::haskell::Cabal::probe,
+    },
+    BuildSystemEntry {
+        name: "gradle",
+        probe: crate::buildsystems::java::Gradle::probe,
+    },
+    BuildSystemEntry {
+        name: "maven",
+        probe: crate::buildsystems::java::Maven::probe,
+    },
+    BuildSystemEntry {
+        name: "distzilla",
+        probe: crate::buildsystems::perl::DistZilla::probe,
+    },
+    BuildSystemEntry {
+        name: "perl-build-tiny",
+        probe: crate::buildsystems::perl::PerlBuildTiny::probe,
+    },
+    BuildSystemEntry {
+        name: "go",
+        probe: crate::buildsystems::go::Golang::probe,
+    },
+    BuildSystemEntry {
+        name: "bazel",
+        probe: crate::buildsystems::bazel::Bazel::probe,
+    },
+    BuildSystemEntry {
+        name: "r",
+        probe: crate::buildsystems::r::R::probe,
+    },
+    BuildSystemEntry {
+        name: "octave",
+        probe: crate::buildsystems::octave::Octave::probe,
+    },
+    BuildSystemEntry {
+        name: "cmake",
+        probe: crate::buildsystems::make::CMake::probe,
+    },
+    BuildSystemEntry {
+        name: "gnome-shell-extension",
+        probe: crate::buildsystems::gnome::GnomeShellExtension::probe,
+    },
+    // Make is intentionally at the end of the list.
+    BuildSystemEntry {
+        name: "make",
+        probe: crate::buildsystems::make::Make::probe,
+    },
+    BuildSystemEntry {
+        name: "composer",
+        probe: Composer::probe,
+    },
+    BuildSystemEntry {
+        name: "runtests",
+        probe: RunTests::probe,
+    },
+];
+
+/// XML namespaces used by PEAR package definitions.
 pub const PEAR_NAMESPACES: &[&str] = &[
     "http://pear.php.net/dtd/package-2.0",
     "http://pear.php.net/dtd/package-2.1",
 ];
 
 #[derive(Debug)]
+/// PEAR (PHP Extension and Application Repository) build system.
 pub struct Pear(pub PathBuf);
 
 impl Pear {
+    /// Create a new PEAR build system.
+    ///
+    /// # Arguments
+    /// * `path` - Path to the PEAR package.xml file
+    ///
+    /// # Returns
+    /// A new PEAR build system instance
     pub fn new(path: PathBuf) -> Self {
         Self(path)
     }
 
+    /// Detect if a directory contains a PEAR project.
+    ///
+    /// # Arguments
+    /// * `path` - Directory to probe
+    ///
+    /// # Returns
+    /// * `Some(Box<dyn BuildSystem>)` if a PEAR project is detected
+    /// * `None` if no PEAR project is detected
     pub fn probe(path: &Path) -> Option<Box<dyn BuildSystem>> {
         let package_xml_path = path.join("package.xml");
         if !package_xml_path.exists() {
@@ -431,11 +644,11 @@ impl BuildSystem for Pear {
 
         let dependencies_tag = root
             .get_child("dependencies")
-            .ok_or_else(|| Error::Other("No <dependencies> tag found in <package>".to_string()))?;
+            .ok_or_else(|| Error::Other("No <dependencies> tag found in <package>".to_owned()))?;
 
         let required_tag = dependencies_tag
             .get_child("required")
-            .ok_or_else(|| Error::Other("No <required> tag found in <dependencies>".to_string()))?;
+            .ok_or_else(|| Error::Other("No <required> tag found in <dependencies>".to_owned()))?;
 
         Ok(required_tag
             .children
@@ -516,13 +729,29 @@ pub fn scan_buildsystems(path: &Path) -> Vec<(PathBuf, Box<dyn BuildSystem>)> {
 }
 
 #[derive(Debug)]
+/// PHP Composer build system.
 pub struct Composer(pub PathBuf);
 
 impl Composer {
+    /// Create a new Composer build system instance.
+    ///
+    /// # Arguments
+    /// * `path` - Path to the project directory
+    ///
+    /// # Returns
+    /// A new Composer build system instance
     pub fn new(path: PathBuf) -> Self {
         Self(path)
     }
 
+    /// Detect if a directory contains a Composer project.
+    ///
+    /// # Arguments
+    /// * `path` - Directory to probe
+    ///
+    /// # Returns
+    /// * `Some(Box<dyn BuildSystem>)` if a Composer project is detected
+    /// * `None` if no Composer project is detected
     pub fn probe(path: &Path) -> Option<Box<dyn BuildSystem>> {
         if path.join("composer.json").exists() {
             Some(Box::new(Self(path.into())))
@@ -590,13 +819,29 @@ impl BuildSystem for Composer {
 }
 
 #[derive(Debug)]
+/// Generic build system that just runs tests.
 pub struct RunTests(pub PathBuf);
 
 impl RunTests {
+    /// Create a new RunTests build system instance.
+    ///
+    /// # Arguments
+    /// * `path` - Path to the project directory
+    ///
+    /// # Returns
+    /// A new RunTests build system instance
     pub fn new(path: PathBuf) -> Self {
         Self(path)
     }
 
+    /// Detect if a directory contains a project with tests that can be run.
+    ///
+    /// # Arguments
+    /// * `path` - Directory to probe
+    ///
+    /// # Returns
+    /// * `Some(Box<dyn BuildSystem>)` if runnable tests are detected
+    /// * `None` if no runnable tests are detected
     pub fn probe(path: &Path) -> Option<Box<dyn BuildSystem>> {
         if path.join("runtests.sh").exists() {
             Some(Box::new(Self(path.into())))
@@ -671,6 +916,16 @@ impl BuildSystem for RunTests {
     }
 }
 
+/// Detect all applicable build systems for a given path.
+///
+/// This function attempts to detect any build systems that can be used with the
+/// provided project directory. Multiple build systems may be detected for a single project.
+///
+/// # Arguments
+/// * `path` - Path to the project directory
+///
+/// # Returns
+/// A vector of detected build systems, sorted in order of preference
 pub fn detect_buildsystems(path: &std::path::Path) -> Vec<Box<dyn BuildSystem>> {
     if !path.exists() {
         log::error!("Path does not exist: {:?}", path);
@@ -678,31 +933,8 @@ pub fn detect_buildsystems(path: &std::path::Path) -> Vec<Box<dyn BuildSystem>>
     }
     let path = path.canonicalize().unwrap();
     let mut ret = vec![];
-    for probe in [
-        Pear::probe,
-        crate::buildsystems::python::SetupPy::probe,
-        crate::buildsystems::node::Node::probe,
-        crate::buildsystems::waf::Waf::probe,
-        crate::buildsystems::ruby::Gem::probe,
-        crate::buildsystems::meson::Meson::probe,
-        crate::buildsystems::rust::Cargo::probe,
-        crate::buildsystems::haskell::Cabal::probe,
-        crate::buildsystems::java::Gradle::probe,
-        crate::buildsystems::java::Maven::probe,
-        crate::buildsystems::perl::DistZilla::probe,
-        crate::buildsystems::perl::PerlBuildTiny::probe,
-        crate::buildsystems::go::Golang::probe,
-        crate::buildsystems::bazel::Bazel::probe,
-        crate::buildsystems::r::R::probe,
-        crate::buildsystems::octave::Octave::probe,
-        crate::buildsystems::make::CMake::probe,
-        crate::buildsystems::gnome::GnomeShellExtension::probe,
-        // Make is intentionally at the end of the list.
-        crate::buildsystems::make::Make::probe,
-        Composer::probe,
-        RunTests::probe,
-    ] {
-        let bs = probe(&path);
+    for entry in BUILDSYSTEM_REGISTRY {
+        let bs = (entry.probe)(&path);
         if let Some(bs) = bs {
             ret.push(bs);
         }
@@ -710,35 +942,47 @@ pub fn detect_buildsystems(path: &std::path::Path) -> Vec<Box<dyn BuildSystem>>
     ret
 }
 
+/// Get the most appropriate build system for a given path.
+///
+/// This function returns the first (most preferred) build system that can be used
+/// with the provided project directory, along with its path.
+///
+/// # Arguments
+/// * `path` - Path to the project directory
+///
+/// # Returns
+/// An optional tuple containing the path to the build system file and the build system instance
 pub fn get_buildsystem(path: &Path) -> Option<(PathBuf, Box<dyn BuildSystem>)> {
     scan_buildsystems(path).into_iter().next()
 }
 
+/// Get all supported build system names.
+///
+/// # Returns
+/// A vector of all supported build system names in detection order
+pub fn supported_buildsystem_names() -> Vec<&'static str> {
+    BUILDSYSTEM_REGISTRY
+        .iter()
+        .map(|entry| entry.name)
+        .collect()
+}
+
+/// Get a build system by name for a given path.
+///
+/// This function tries to create a specific build system by name for the provided
+/// project directory.
+///
+/// # Arguments
+/// * `name` - Name of the build system to use
+/// * `path` - Path to the project directory
+///
+/// # Returns
+/// An optional build system instance if the specified build system is applicable
 pub fn buildsystem_by_name(name: &str, path: &Path) -> Option<Box<dyn BuildSystem>> {
-    match name {
-        "pear" => Pear::probe(path),
-        "composer" => Composer::probe(path),
-        "runtests" => RunTests::probe(path),
-        "setup.py" => crate::buildsystems::python::SetupPy::probe(path),
-        "node" => crate::buildsystems::node::Node::probe(path),
-        "waf" => crate::buildsystems::waf::Waf::probe(path),
-        "gem" => crate::buildsystems::ruby::Gem::probe(path),
-        "meson" => crate::buildsystems::meson::Meson::probe(path),
-        "cargo" => crate::buildsystems::rust::Cargo::probe(path),
-        "cabal" => crate::buildsystems::haskell::Cabal::probe(path),
-        "gradle" => crate::buildsystems::java::Gradle::probe(path),
-        "maven" => crate::buildsystems::java::Maven::probe(path),
-        "distzilla" => crate::buildsystems::perl::DistZilla::probe(path),
-        "perl-build-tiny" => crate::buildsystems::perl::PerlBuildTiny::probe(path),
-        "go" => crate::buildsystems::go::Golang::probe(path),
-        "bazel" => crate::buildsystems::bazel::Bazel::probe(path),
-        "r" => crate::buildsystems::r::R::probe(path),
-        "octave" => crate::buildsystems::octave::Octave::probe(path),
-        "cmake" => crate::buildsystems::make::CMake::probe(path),
-        "gnome-shell-extension" => crate::buildsystems::gnome::GnomeShellExtension::probe(path),
-        "make" => crate::buildsystems::make::Make::probe(path),
-        _ => None,
-    }
+    BUILDSYSTEM_REGISTRY
+        .iter()
+        .find(|entry| entry.name == name)
+        .and_then(|entry| (entry.probe)(path))
 }
 
 #[cfg(test)]
@@ -765,4 +1009,14 @@ mod tests {
             InstallerError::UnknownDependencyFamily,
         ));
     }
+
+    #[test]
+    fn test_supported_buildsystem_names() {
+        let names = supported_buildsystem_names();
+        assert!(!names.is_empty());
+        assert!(names.contains(&"cargo"));
+        assert!(names.contains(&"make"));
+        assert!(names.contains(&"meson"));
+        assert_eq!(names.len(), 21);
+    }
 }
