@@ -1,3 +1,8 @@
+//! Support for Python build systems.
+//!
+//! This module provides functionality for building, testing, and installing
+//! Python packages using various build systems such as setuptools, poetry, and pip.
+
 use crate::analyze::{run_detecting_problems, AnalyzedError};
 use crate::buildsystem::{BuildSystem, DependencyCategory, Error, InstallTarget};
 use crate::dependencies::python::{PythonDependency, PythonPackageDependency};
@@ -33,12 +38,37 @@ struct Distribution {
 
 fn load_toml(path: &Path) -> Result<pyproject_toml::PyProjectToml, PyErr> {
     let path = path.join("pyproject.toml");
-    let text = std::fs::read_to_string(path).unwrap();
+    let text = match std::fs::read_to_string(&path) {
+        Ok(text) => text,
+        Err(e) => {
+            return Err(match e.kind() {
+                std::io::ErrorKind::NotFound => {
+                    PyFileNotFoundError::new_err(format!("File not found: {}", path.display()))
+                }
+                _ => pyo3::exceptions::PyIOError::new_err(format!(
+                    "Failed to read {}: {}",
+                    path.display(),
+                    e
+                )),
+            })
+        }
+    };
 
-    Ok(toml::from_str(&text).unwrap())
+    match toml::from_str(&text) {
+        Ok(parsed) => Ok(parsed),
+        Err(e) => Err(pyo3::exceptions::PyValueError::new_err(format!(
+            "Failed to parse {}: {}",
+            path.display(),
+            e
+        ))),
+    }
 }
 
 #[derive(Debug)]
+/// A wrapper around a Python setup.cfg configuration file.
+///
+/// This provides access to the configuration in a setup.cfg file, which is used
+/// by setuptools to configure Python package builds.
 pub struct SetupCfg(PyObject);
 
 impl SetupCfg {
@@ -61,7 +91,7 @@ impl SetupCfg {
                     .unwrap()
                     .extract(py)
                     .ok();
-                Some(SetupCfgSection(section.to_object(py)))
+                Some(SetupCfgSection(section.unwrap()))
             } else {
                 None
             }
@@ -69,6 +99,10 @@ impl SetupCfg {
     }
 }
 
+/// A section in a Python setup.cfg configuration file.
+///
+/// This provides access to a specific section in a setup.cfg file, allowing
+/// access to configuration keys within that section.
 pub struct SetupCfgSection(PyObject);
 
 impl Default for SetupCfg {
@@ -88,6 +122,13 @@ impl SetupCfgSection {
         })
     }
 
+    /// Check if a key exists in this section.
+    ///
+    /// # Arguments
+    /// * `key` - The key to check for
+    ///
+    /// # Returns
+    /// `true` if the key exists, `false` otherwise
     pub fn has_key(&self, key: &str) -> bool {
         Python::with_gil(|py| {
             self.0
@@ -101,14 +142,14 @@ impl SetupCfgSection {
 
 fn load_setup_cfg(path: &Path) -> Result<Option<SetupCfg>, PyErr> {
     Python::with_gil(|py| {
-        let m = py.import_bound("setuptools.config.setupcfg")?;
+        let m = py.import("setuptools.config.setupcfg")?;
         let read_configuration = m.getattr("read_configuration")?;
 
         let p = path.join("setup.cfg");
 
         if p.exists() {
             let config = read_configuration.call1((p,))?;
-            Ok(Some(SetupCfg(config.to_object(py))))
+            Ok(Some(SetupCfg(config.unbind())))
         } else {
             Ok(None)
         }
@@ -126,28 +167,25 @@ fn run_setup(py: Python, script_name: &Path, stop_after: &str) -> PyResult<PyObj
             || stop_after == "run"
     );
     // Import setuptools, just in case it decides to replace distutils
-    let _ = py.import_bound("setuptools");
+    let _ = py.import("setuptools");
 
-    let core = match py.import_bound("distutils.core") {
+    let core = match py.import("distutils.core") {
         Ok(m) => m,
         Err(e) if e.is_instance_of::<PyImportError>(py) => {
             // Importing distutils failed, but that's fine.
-            match py.import_bound("setuptools._distutils.core") {
-                Ok(m) => m,
-                Err(e) => return Err(e),
-            }
+            py.import("setuptools._distutils.core")?
         }
         Err(e) => return Err(e),
     };
 
     core.setattr("_setup_stop_after", stop_after)?;
 
-    let sys = py.import_bound("sys")?;
-    let os = py.import_bound("os")?;
+    let sys = py.import("sys")?;
+    let os = py.import("os")?;
 
     let save_argv = sys.getattr("argv")?;
 
-    let g = PyDict::new_bound(py);
+    let g = PyDict::new(py);
     g.set_item("__file__", script_name)?;
     g.set_item("__name__", "__main")?;
 
@@ -162,16 +200,17 @@ fn run_setup(py: Python, script_name: &Path, stop_after: &str) -> PyResult<PyObj
 
     let text = std::fs::read_to_string(script_name)?;
 
-    let r = py.eval_bound(text.as_str(), Some(&g), None);
+    let code = std::ffi::CString::new(text).unwrap();
+    let r = py.eval(&code, Some(&g), None);
 
     os.call_method1("chdir", (old_cwd,))?;
     sys.setattr("argv", save_argv)?;
     core.setattr("_setup_stop_after", py.None())?;
 
     match r {
-        Ok(_) => Ok(core.getattr("_setup_distribution")?.to_object(py)),
+        Ok(_) => Ok(core.getattr("_setup_distribution")?.unbind()),
         Err(e) if e.is_instance_of::<PySystemExit>(py) => {
-            Ok(core.getattr("_setup_distribution")?.to_object(py))
+            Ok(core.getattr("_setup_distribution")?.unbind())
         }
         Err(e) => Err(e),
     }
@@ -228,6 +267,10 @@ with open(%(output_path)s, 'w') as f:
 """#;
 
 #[derive(Debug)]
+/// A Python setuptools-based build system.
+///
+/// This build system handles Python packages that use setup.py for building,
+/// which is the traditional approach for Python packages.
 pub struct SetupPy {
     path: PathBuf,
     has_setup_py: bool,
@@ -238,6 +281,15 @@ pub struct SetupPy {
 }
 
 impl SetupPy {
+    /// Create a new SetupPy build system with the specified path.
+    ///
+    /// This will load and parse setup.cfg and pyproject.toml if they exist.
+    ///
+    /// # Arguments
+    /// * `path` - The path to the Python project directory
+    ///
+    /// # Returns
+    /// A new SetupPy build system instance
     pub fn new(path: &Path) -> Self {
         let has_setup_py = path.join("setup.py").exists();
 
@@ -278,6 +330,13 @@ impl SetupPy {
         })
     }
 
+    /// Probe a directory for a Python setuptools build system.
+    ///
+    /// # Arguments
+    /// * `path` - The path to check
+    ///
+    /// # Returns
+    /// A SetupPy build system if one exists at the path, `None` otherwise
     pub fn probe(path: &Path) -> Option<Box<dyn BuildSystem>> {
         if path.join("setup.py").exists() {
             log::debug!("Found setup.py, assuming python project.");
@@ -837,3 +896,64 @@ impl BuildSystem for SetupPy {
         self
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use std::fs;
+    use tempfile::tempdir;
+
+    #[test]
+    fn test_python_project_without_pyproject_toml() {
+        pyo3::prepare_freethreaded_python();
+
+        let temp_dir = tempdir().unwrap();
+        let path = temp_dir.path();
+
+        // Create only setup.py, no pyproject.toml
+        fs::write(
+            path.join("setup.py"),
+            "from setuptools import setup\nsetup(name='test')",
+        )
+        .unwrap();
+
+        // This should not panic
+        let setup_py = SetupPy::new(path);
+        assert!(setup_py.has_setup_py);
+        assert!(setup_py.pyproject.is_none());
+    }
+
+    #[test]
+    fn test_load_toml_file_not_found() {
+        pyo3::prepare_freethreaded_python();
+        Python::with_gil(|py| {
+            let temp_dir = tempdir().unwrap();
+            let path = temp_dir.path();
+
+            // Don't create pyproject.toml
+            let result = load_toml(path);
+            assert!(result.is_err());
+
+            let err = result.unwrap_err();
+            assert!(err.is_instance_of::<PyFileNotFoundError>(py));
+        });
+    }
+
+    #[test]
+    fn test_load_toml_invalid_content() {
+        pyo3::prepare_freethreaded_python();
+        Python::with_gil(|py| {
+            let temp_dir = tempdir().unwrap();
+            let path = temp_dir.path();
+
+            // Create invalid pyproject.toml
+            fs::write(path.join("pyproject.toml"), "invalid toml content").unwrap();
+
+            let result = load_toml(path);
+            assert!(result.is_err());
+
+            let err = result.unwrap_err();
+            assert!(err.is_instance_of::<pyo3::exceptions::PyValueError>(py));
+        });
+    }
+}
