@@ -1,10 +1,26 @@
+//! Debian package building functionality.
+//!
+//! This module provides utilities for building Debian packages, including
+//! functions for managing changelog entries, running build commands,
+//! and handling build failures.
+
 use breezyshim::tree::{MutableTree, Tree, WorkingTree};
+use breezyshim::workingtree::PyWorkingTree;
 use buildlog_consultant::Problem;
 use debian_changelog::{ChangeLog, Urgency};
 use debversion::Version;
 use std::io::Seek;
 use std::path::{Path, PathBuf};
 
+/// Get the current Debian build architecture.
+///
+/// Uses dpkg-architecture to determine the build architecture.
+///
+/// # Returns
+/// The build architecture string (e.g., "amd64")
+///
+/// # Panics
+/// Panics if dpkg-architecture is not available or fails to run.
 pub fn get_build_architecture() -> String {
     std::process::Command::new("dpkg-architecture")
         .arg("-qDEB_BUILD_ARCH")
@@ -13,12 +29,21 @@ pub fn get_build_architecture() -> String {
         .unwrap()
 }
 
+/// Default build command for Debian packages.
+///
+/// Uses sbuild with the --no-clean-source option.
 pub const DEFAULT_BUILDER: &str = "sbuild --no-clean-source";
 
+/// Get the path to the current Python interpreter.
+///
+/// Uses PyO3 to get the path from sys.executable.
+///
+/// # Returns
+/// The path to the Python interpreter as a String.
 fn python_command() -> String {
     pyo3::Python::with_gil(|py| {
         use pyo3::types::PyAnyMethods;
-        let sys_module = py.import_bound("sys").unwrap();
+        let sys_module = py.import("sys").unwrap();
         sys_module
             .getattr("executable")
             .unwrap()
@@ -27,6 +52,17 @@ fn python_command() -> String {
     })
 }
 
+/// Generate the command for building Debian packages with bzr-builddeb.
+///
+/// # Arguments
+/// * `build_command` - Custom build command to use (defaults to DEFAULT_BUILDER)
+/// * `result_dir` - Directory to store build results
+/// * `apt_repository` - APT repository to use
+/// * `apt_repository_key` - APT repository key to use
+/// * `extra_repositories` - Additional APT repositories
+///
+/// # Returns
+/// Vector of command line arguments for the build command
 pub fn builddeb_command(
     build_command: Option<&str>,
     result_dir: Option<&std::path::Path>,
@@ -67,6 +103,7 @@ pub fn builddeb_command(
     args
 }
 
+/// Error indicating a build failure with an exit code.
 #[derive(Debug)]
 pub struct BuildFailedError(pub i32);
 
@@ -78,8 +115,24 @@ impl std::fmt::Display for BuildFailedError {
 
 impl std::error::Error for BuildFailedError {}
 
+/// Build a Debian package from source.
+///
+/// # Arguments
+/// * `local_tree` - Working tree containing the package source
+/// * `outf` - File to write build output to
+/// * `build_command` - Command to use for building
+/// * `result_dir` - Directory to store build results
+/// * `distribution` - Debian distribution to target
+/// * `subpath` - Path within the tree where the package is located
+/// * `source_date_epoch` - Source date epoch to use for reproducible builds
+/// * `apt_repository` - APT repository to use
+/// * `apt_repository_key` - APT repository key to use
+/// * `extra_repositories` - Additional APT repositories
+///
+/// # Returns
+/// Ok on success, BuildFailedError with exit code on failure
 pub fn build(
-    local_tree: &WorkingTree,
+    local_tree: &dyn PyWorkingTree,
     outf: std::fs::File,
     build_command: &str,
     result_dir: Option<&std::path::Path>,
@@ -134,35 +187,78 @@ pub fn build(
     }
 }
 
+/// Default filename for build logs.
 pub const BUILD_LOG_FILENAME: &str = "build.log";
 
 #[derive(Debug)]
+/// Error that can occur during a build attempt.
+///
+/// Contains details about the build failure, including stage, phase,
+/// return code, command, and error information.
 pub enum BuildOnceError {
+    /// Detailed error with specific problem information.
     Detailed {
+        /// Build stage where failure occurred.
         stage: Option<String>,
+        /// Build phase where failure occurred.
         phase: Option<buildlog_consultant::sbuild::Phase>,
+        /// Process return code.
         retcode: i32,
+        /// Command that was run.
         command: Vec<String>,
+        /// Specific error that was detected.
         error: Box<dyn Problem>,
+        /// Human-readable description of the error.
         description: String,
     },
+    /// Unidentified error without specific problem information.
     Unidentified {
+        /// Build stage where failure occurred.
         stage: Option<String>,
+        /// Build phase where failure occurred.
         phase: Option<buildlog_consultant::sbuild::Phase>,
+        /// Process return code.
         retcode: i32,
+        /// Command that was run.
         command: Vec<String>,
+        /// Human-readable description of the error.
         description: String,
     },
 }
 
+/// Result of a successful build attempt.
+///
+/// Contains information about the built package, including source package name,
+/// version, and paths to changes files.
 pub struct BuildOnceResult {
+    /// Name of the source package.
     pub source_package: String,
+    /// Version of the built package.
     pub version: debversion::Version,
+    /// Paths to the generated .changes files.
     pub changes_names: Vec<PathBuf>,
 }
 
+/// Build a Debian package once and capture detailed error information.
+///
+/// This function builds a package and provides more detailed error information
+/// than the basic `build` function by parsing the build log.
+///
+/// # Arguments
+/// * `local_tree` - Working tree containing the package source
+/// * `build_suite` - Debian distribution to target
+/// * `output_directory` - Directory to store build results
+/// * `build_command` - Command to use for building
+/// * `subpath` - Path within the tree where the package is located
+/// * `source_date_epoch` - Source date epoch to use for reproducible builds
+/// * `apt_repository` - APT repository to use
+/// * `apt_repository_key` - APT repository key to use
+/// * `extra_repositories` - Additional APT repositories
+///
+/// # Returns
+/// BuildOnceResult on success, detailed BuildOnceError on failure
 pub fn build_once(
-    local_tree: &WorkingTree,
+    local_tree: &dyn PyWorkingTree,
     build_suite: Option<&str>,
     output_directory: &Path,
     build_command: &str,
@@ -247,7 +343,15 @@ pub fn build_once(
     })
 }
 
-fn control_files_in_root(tree: &dyn MutableTree, subpath: &std::path::Path) -> bool {
+/// Check if Debian control files are in the root of the project.
+///
+/// # Arguments
+/// * `tree` - Tree to check for control files
+/// * `subpath` - Path within the tree to check
+///
+/// # Returns
+/// true if control files are in root, false if they are in a debian/ directory
+fn control_files_in_root(tree: &dyn PyWorkingTree, subpath: &std::path::Path) -> bool {
     let debian_path = subpath.join("debian");
     if tree.has_filename(&debian_path) {
         return false;
@@ -262,8 +366,16 @@ fn control_files_in_root(tree: &dyn MutableTree, subpath: &std::path::Path) -> b
     )))
 }
 
+/// Get the last entry from the debian/changelog file.
+///
+/// # Arguments
+/// * `local_tree` - Working tree containing the package source
+/// * `subpath` - Path within the tree where the package is located
+///
+/// # Returns
+/// Tuple of (package name, version) from the last changelog entry
 fn get_last_changelog_entry(
-    local_tree: &WorkingTree,
+    local_tree: &dyn PyWorkingTree,
     subpath: &std::path::Path,
 ) -> (String, debversion::Version) {
     let path = if control_files_in_root(local_tree, subpath) {
@@ -281,6 +393,13 @@ fn get_last_changelog_entry(
     (e.package().unwrap(), e.version().unwrap())
 }
 
+/// Run gbp-dch to update the changelog.
+///
+/// # Arguments
+/// * `path` - Path to the package directory
+///
+/// # Returns
+/// Ok on success, Error if gbp-dch fails
 pub fn gbp_dch(path: &std::path::Path) -> Result<(), std::io::Error> {
     let cmd = std::process::Command::new("gbp-dch")
         .arg("--ignore-branch")
@@ -295,6 +414,15 @@ pub fn gbp_dch(path: &std::path::Path) -> Result<(), std::io::Error> {
     Ok(())
 }
 
+/// Find changes files for a specific package and version.
+///
+/// # Arguments
+/// * `path` - Directory to search for changes files
+/// * `package` - Package name to match
+/// * `version` - Version to match
+///
+/// # Returns
+/// Iterator of (architecture, DirEntry) pairs for matching changes files
 pub fn find_changes_files(
     path: &std::path::Path,
     package: &str,
@@ -334,7 +462,7 @@ pub fn find_changes_files(
 /// * `apt_repository_key` - APT repository key to use.
 /// * `extra_repositories` - Extra repositories to use.
 pub fn attempt_build(
-    local_tree: &WorkingTree,
+    local_tree: &dyn PyWorkingTree,
     suffix: Option<&str>,
     build_suite: Option<&str>,
     output_directory: &std::path::Path,
@@ -349,14 +477,9 @@ pub fn attempt_build(
 ) -> Result<BuildOnceResult, BuildOnceError> {
     if run_gbp_dch
         && subpath == std::path::Path::new("")
-        && pyo3::Python::with_gil(|py| {
-            use pyo3::ToPyObject;
-            local_tree
-                .controldir()
-                .to_object(py)
-                .getattr(py, "_git")
-                .is_ok()
-        })
+        && local_tree
+            .abspath(std::path::Path::new(".git"))
+            .map_or(false, |p| std::path::Path::new(&p).exists())
     {
         gbp_dch(&local_tree.abspath(subpath).unwrap()).unwrap();
     }
@@ -392,6 +515,16 @@ pub fn attempt_build(
     )
 }
 
+/// Add a suffix to a version string.
+///
+/// If the version already has the same suffix, increments the number at the end.
+///
+/// # Arguments
+/// * `version` - Version to modify
+/// * `suffix` - Suffix to add to the version
+///
+/// # Returns
+/// New version with the suffix added or incremented
 pub fn version_add_suffix(version: &Version, suffix: &str) -> Version {
     fn add_suffix(v: &str, suffix: &str) -> String {
         if let Some(m) = regex::Regex::new(&format!("(.*)({})([0-9]+)", regex::escape(suffix)))
@@ -431,7 +564,7 @@ pub fn version_add_suffix(version: &Version, suffix: &str) -> Version {
 /// # Returns
 /// The version of the newly added entry.
 pub fn add_dummy_changelog_entry(
-    tree: &dyn MutableTree,
+    tree: &dyn PyWorkingTree,
     subpath: &Path,
     suffix: &str,
     suite: &str,
@@ -470,13 +603,113 @@ pub fn add_dummy_changelog_entry(
 mod tests {
     use super::*;
     use breezyshim::tree::MutableTree;
+    use std::fs::File;
+    use tempfile::tempdir;
 
     #[test]
+    #[cfg(target_os = "linux")]
     fn test_get_build_architecture() {
         let arch = get_build_architecture();
         assert!(!arch.is_empty() && arch.len() < 10);
     }
 
+    #[test]
+    #[cfg(not(target_os = "linux"))]
+    #[should_panic(expected = "No such file or directory")]
+    fn test_get_build_architecture_panics_on_non_linux() {
+        // This test verifies that the function panics when dpkg-architecture isn't available
+        let _ = get_build_architecture();
+    }
+
+    #[test]
+    fn test_build_fails_with_invalid_command() {
+        // Set up a test environment
+        let td = tempdir().unwrap();
+        let tree = breezyshim::controldir::create_standalone_workingtree(
+            td.path(),
+            &breezyshim::controldir::ControlDirFormat::default(),
+        )
+        .unwrap();
+
+        // Create a file to capture output
+        let log_file = File::create(td.path().join("build.log")).unwrap();
+
+        // Test with a command that will definitely fail
+        let result = build(
+            &tree,
+            log_file,
+            "command_that_does_not_exist",
+            None,
+            None,
+            Path::new(""),
+            None,
+            None,
+            None,
+            None,
+        );
+
+        // Verify the build fails
+        assert!(result.is_err());
+        if let Err(BuildFailedError(code)) = result {
+            // The code can be 1 on Linux or other values on different platforms
+            assert!(code > 0);
+        } else {
+            panic!("Expected BuildFailedError");
+        }
+    }
+
+    #[test]
+    fn test_build_once_error_conversion() {
+        // Test conversion between BuildFailedError and BuildOnceError
+        // Set up a test environment
+        let td = tempdir().unwrap();
+        let tree = breezyshim::controldir::create_standalone_workingtree(
+            td.path(),
+            &breezyshim::controldir::ControlDirFormat::default(),
+        )
+        .unwrap();
+
+        // Create a stub debian directory structure
+        std::fs::create_dir_all(td.path().join("debian")).unwrap();
+
+        // Create minimal changelog file
+        std::fs::write(
+            td.path().join("debian/changelog"),
+            r#"test-package (1.0) unstable; urgency=low
+            
+  * Initial release
+            
+ -- Test User <test@example.com>  Thu, 01 Jan 2023 00:00:00 +0000
+"#,
+        )
+        .unwrap();
+
+        tree.add(&[Path::new("debian"), Path::new("debian/changelog")])
+            .unwrap();
+
+        // Build will fail but we're testing error conversion
+        let output_dir = tempdir().unwrap();
+        let result = build_once(
+            &tree,
+            None,
+            output_dir.path(),
+            "command_that_does_not_exist",
+            Path::new(""),
+            None,
+            None,
+            None,
+            None,
+        );
+
+        assert!(result.is_err());
+        match result {
+            Err(BuildOnceError::Unidentified { .. }) | Err(BuildOnceError::Detailed { .. }) => {
+                // Success - proper error conversion happened
+            }
+            _ => panic!("Expected Unidentified or Detailed BuildOnceError"),
+        }
+    }
+
     #[test]
     fn test_builddeb_command() {
         let command = builddeb_command(
