@@ -2,15 +2,19 @@ use crate::dependency::Dependency;
 use crate::installer::{Error, Explanation, InstallationScope, Installer};
 use crate::session::Session;
 use serde::{Deserialize, Serialize};
-use std::path::Path;
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
+/// Represents a Go package dependency.
 pub struct GoPackageDependency {
+    /// The name of the Go package.
     pub package: String,
+
+    /// The version of the Go package, if specified.
     pub version: Option<String>,
 }
 
 impl GoPackageDependency {
+    /// Creates a new `GoPackageDependency` instance.
     pub fn new(package: &str, version: Option<&str>) -> Self {
         Self {
             package: package.to_string(),
@@ -18,6 +22,7 @@ impl GoPackageDependency {
         }
     }
 
+    /// Creates a simple `GoPackageDependency` instance without a version.
     pub fn simple(package: &str) -> Self {
         Self {
             package: package.to_string(),
@@ -26,6 +31,65 @@ impl GoPackageDependency {
     }
 }
 
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::buildlog::ToDependency;
+    use std::any::Any;
+
+    #[test]
+    fn test_go_package_dependency_new() {
+        let dependency = GoPackageDependency::new("github.com/pkg/errors", Some("v0.9.1"));
+        assert_eq!(dependency.package, "github.com/pkg/errors");
+        assert_eq!(dependency.version, Some("v0.9.1".to_string()));
+    }
+
+    #[test]
+    fn test_go_package_dependency_simple() {
+        let dependency = GoPackageDependency::simple("github.com/pkg/errors");
+        assert_eq!(dependency.package, "github.com/pkg/errors");
+        assert_eq!(dependency.version, None);
+    }
+
+    #[test]
+    fn test_go_package_dependency_family() {
+        let dependency = GoPackageDependency::simple("github.com/pkg/errors");
+        assert_eq!(dependency.family(), "go-package");
+    }
+
+    #[test]
+    fn test_go_package_dependency_as_any() {
+        let dependency = GoPackageDependency::simple("github.com/pkg/errors");
+        let any_dep: &dyn Any = dependency.as_any();
+        assert!(any_dep.downcast_ref::<GoPackageDependency>().is_some());
+    }
+
+    #[test]
+    fn test_missing_go_package_to_dependency() {
+        let problem = buildlog_consultant::problems::common::MissingGoPackage {
+            package: "github.com/pkg/errors".to_string(),
+        };
+        let dependency = problem.to_dependency();
+        assert!(dependency.is_some());
+        let dep = dependency.unwrap();
+        assert_eq!(dep.family(), "go-package");
+        let go_dep = dep.as_any().downcast_ref::<GoPackageDependency>().unwrap();
+        assert_eq!(go_dep.package, "github.com/pkg/errors");
+    }
+
+    #[test]
+    fn test_go_dependency_new() {
+        let dependency = GoDependency::new(Some("1.16"));
+        assert_eq!(dependency.version, Some("1.16".to_string()));
+    }
+
+    #[test]
+    fn test_go_dependency_family() {
+        let dependency = GoDependency::new(None);
+        assert_eq!(dependency.family(), "go");
+    }
+}
+
 impl Dependency for GoPackageDependency {
     fn family(&self) -> &'static str {
         "go-package"
@@ -62,6 +126,8 @@ impl crate::dependencies::debian::IntoDebianDependency for GoPackageDependency {
         &self,
         apt: &crate::debian::apt::AptManager,
     ) -> std::option::Option<std::vec::Vec<crate::dependencies::debian::DebianDependency>> {
+        use std::path::Path;
+
         let names = apt
             .get_packages_for_paths(
                 vec![Path::new("/usr/share/gocode/src")
@@ -118,11 +184,14 @@ impl crate::buildlog::ToDependency for buildlog_consultant::problems::common::Mi
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
+/// Represents a Go dependency.
 pub struct GoDependency {
+    /// The version of the Go dependency, if specified.
     pub version: Option<String>,
 }
 
 impl GoDependency {
+    /// Creates a new `GoDependency` instance.
     pub fn new(version: Option<&str>) -> Self {
         Self {
             version: version.map(|s| s.to_string()),
@@ -164,11 +233,13 @@ impl crate::upstream::FindUpstream for GoPackageDependency {
     }
 }
 
+/// A resolver for Go package dependencies.
 pub struct GoResolver<'a> {
     session: &'a dyn Session,
 }
 
 impl<'a> GoResolver<'a> {
+    /// Creates a new `GoResolver` instance.
     pub fn new(session: &'a dyn Session) -> Self {
         Self { session }
     }
@@ -194,7 +265,7 @@ impl<'a> Installer for GoResolver<'a> {
             .ok_or(Error::UnknownDependencyFamily)?;
         Ok(Explanation {
             message: format!("Install go package {}", req.package),
-            command: Some(self.cmd(&[&req])),
+            command: Some(self.cmd(&[req])),
         })
     }
 
@@ -203,7 +274,7 @@ impl<'a> Installer for GoResolver<'a> {
             .as_any()
             .downcast_ref::<GoPackageDependency>()
             .ok_or(Error::UnknownDependencyFamily)?;
-        let cmd = self.cmd(&[&req]);
+        let cmd = self.cmd(&[req]);
         let (env, user) = match scope {
             InstallationScope::User => (std::collections::HashMap::new(), None),
             InstallationScope::Global => {
