@@ -4,12 +4,14 @@ use crate::session::Session;
 use serde::{Deserialize, Serialize};
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
+/// A dependency on a Haskell package
 pub struct HaskellPackageDependency {
     package: String,
     specs: Option<Vec<String>>,
 }
 
 impl HaskellPackageDependency {
+    /// Creates a new HaskellPackageDependency
     pub fn new(package: &str, specs: Option<Vec<&str>>) -> Self {
         Self {
             package: package.to_string(),
@@ -17,6 +19,7 @@ impl HaskellPackageDependency {
         }
     }
 
+    /// Creates a new HaskellPackageDependency with no specs
     pub fn simple(package: &str) -> Self {
         Self {
             package: package.to_string(),
@@ -105,11 +108,13 @@ impl crate::dependencies::debian::IntoDebianDependency for HaskellPackageDepende
     }
 }
 
+/// A resolver for Haskell packages using the `cabal` command
 pub struct HackageResolver<'a> {
     session: &'a dyn Session,
 }
 
 impl<'a> HackageResolver<'a> {
+    /// Creates a new HackageResolver
     pub fn new(session: &'a dyn Session) -> Self {
         Self { session }
     }
@@ -191,6 +196,64 @@ impl crate::buildlog::ToDependency
 impl crate::upstream::FindUpstream for HaskellPackageDependency {
     fn find_upstream(&self) -> Option<crate::upstream::UpstreamMetadata> {
         let rt = tokio::runtime::Runtime::new().unwrap();
-        rt.block_on(upstream_ontologist::providers::haskell::remote_hackage_data(&self.package)).ok()
+        rt.block_on(upstream_ontologist::providers::haskell::remote_hackage_data(&self.package))
+            .ok()
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::buildlog::ToDependency;
+    use std::str::FromStr;
+
+    #[test]
+    fn test_haskell_package_dependency_new() {
+        let dependency = HaskellPackageDependency::new("parsec", Some(vec![">=3.1.11"]));
+        assert_eq!(dependency.package, "parsec");
+        assert_eq!(dependency.specs, Some(vec![">=3.1.11".to_string()]));
+    }
+
+    #[test]
+    fn test_haskell_package_dependency_simple() {
+        let dependency = HaskellPackageDependency::simple("parsec");
+        assert_eq!(dependency.package, "parsec");
+        assert_eq!(dependency.specs, None);
+    }
+
+    #[test]
+    fn test_haskell_package_dependency_family() {
+        let dependency = HaskellPackageDependency::simple("parsec");
+        assert_eq!(dependency.family(), "haskell-package");
+    }
+
+    #[test]
+    fn test_haskell_package_dependency_as_any() {
+        let dependency = HaskellPackageDependency::simple("parsec");
+        let any_dep = dependency.as_any();
+        assert!(any_dep.downcast_ref::<HaskellPackageDependency>().is_some());
+    }
+
+    #[test]
+    fn test_haskell_package_dependency_from_str() {
+        let dependency = HaskellPackageDependency::from_str("parsec >=3.1.11").unwrap();
+        assert_eq!(dependency.package, "parsec");
+        assert_eq!(dependency.specs, Some(vec![">=3.1.11".to_string()]));
+    }
+
+    #[test]
+    fn test_missing_haskell_dependencies_to_dependency() {
+        let problem = buildlog_consultant::problems::common::MissingHaskellDependencies(vec![
+            "parsec".to_string(),
+        ]);
+        let dependency = problem.to_dependency();
+        assert!(dependency.is_some());
+        let dep = dependency.unwrap();
+        assert_eq!(dep.family(), "haskell-package");
+        let haskell_dep = dep
+            .as_any()
+            .downcast_ref::<HaskellPackageDependency>()
+            .unwrap();
+        assert_eq!(haskell_dep.package, "parsec");
     }
 }
