@@ -1,5 +1,9 @@
 use crate::session::{CommandBuilder, Error, Project, Session};
 
+/// A session implementation that runs commands on the local system.
+///
+/// This is the simplest session implementation, which just runs commands
+/// directly on the host system without any isolation.
 pub struct PlainSession(std::path::PathBuf);
 
 impl Default for PlainSession {
@@ -9,6 +13,13 @@ impl Default for PlainSession {
 }
 
 impl PlainSession {
+    /// Create a new PlainSession.
+    ///
+    /// This creates a new session with the root directory (/) as the
+    /// current working directory.
+    ///
+    /// # Returns
+    /// A new PlainSession instance
     pub fn new() -> Self {
         PlainSession(std::path::PathBuf::from("/"))
     }
@@ -168,7 +179,7 @@ impl Session for PlainSession {
         include_controldir: Option<bool>,
         subdir: Option<&str>,
     ) -> Result<Project, Error> {
-        use crate::vcs::{dupe_vcs_tree, export_vcs_tree};
+        use crate::vcs::dupe_vcs_tree;
         if include_controldir.unwrap_or(true) && tree.basedir().is_some() {
             // Optimization: just use the directory as-is, don't copy anything
             Ok(Project::Noop(tree.basedir().unwrap()))
@@ -179,11 +190,11 @@ impl Session for PlainSession {
             } else {
                 td.path().to_path_buf()
             };
-            export_vcs_tree(tree.as_tree(), &p, None).unwrap();
+            tree.export_to(&p, None).unwrap();
             Ok(Project::Temporary {
                 internal_path: p.clone(),
                 external_path: p,
-                td: td.into_path(),
+                td: td.keep(),
             })
         } else {
             let td = tempfile::tempdir().unwrap();
@@ -196,7 +207,7 @@ impl Session for PlainSession {
             Ok(Project::Temporary {
                 internal_path: p.clone(),
                 external_path: p,
-                td: td.into_path(),
+                td: td.keep(),
             })
         }
     }
@@ -216,6 +227,7 @@ impl Session for PlainSession {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use breezyshim::WorkingTree;
 
     #[test]
     fn test_prepend_user() {
@@ -380,18 +392,24 @@ mod tests {
             .unwrap();
         tree.add(&[std::path::Path::new("test")]).unwrap();
         tree.build_commit().message("test").commit().unwrap();
-        let project = session.project_from_vcs(&tree, None, None).unwrap();
+        let project = session
+            .project_from_vcs(&tree as &dyn crate::vcs::DupableTree, None, None)
+            .unwrap();
         assert_eq!(project.external_path(), path.canonicalize().unwrap());
         assert_eq!(project.internal_path(), path.canonicalize().unwrap());
         assert!(project.external_path().join(".bzr").exists());
 
-        let project = session.project_from_vcs(&tree, Some(true), None).unwrap();
+        let project = session
+            .project_from_vcs(&tree as &dyn crate::vcs::DupableTree, Some(true), None)
+            .unwrap();
         assert_eq!(project.external_path(), path.canonicalize().unwrap());
         assert_eq!(project.internal_path(), path.canonicalize().unwrap());
 
         assert!(project.external_path().join(".bzr").exists());
 
-        let project = session.project_from_vcs(&tree, Some(false), None).unwrap();
+        let project = session
+            .project_from_vcs(&tree as &dyn crate::vcs::DupableTree, Some(false), None)
+            .unwrap();
         assert_ne!(project.external_path(), path.canonicalize().unwrap());
         assert_ne!(project.internal_path(), path.canonicalize().unwrap());
 
